﻿package com.josephlabrecque {
	
	import flash.display.Sprite;
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.ui.Mouse;
	import flash.utils.getTimer;
	import flash.system.System;
	
	import com.josephlabrecque.actors.Player;
	import com.josephlabrecque.actors.Obstacle;
	import com.josephlabrecque.events.PlayerEvent;
	import com.josephlabrecque.events.ObstacleEvent;
	import com.josephlabrecque.utils.GameSounds;
	
	public class Game extends Sprite {
		
		private const OBSTACLE_COUNT:int = 30;
		private var _state:String;
		private var _obstacleIndex:int;
		private var _playTimeStart:int;
		private var _laserFired:Boolean;
		
		public var introScreen:IntroScreen;
		public var loseScreen:LoseScreen;
		public var scoreKeeper:ScoreKeeper;
		public var bg1:Background;
		public var bg2:Background;
		public var player:Player;
		public var bg:Sprite;
		public var laser:Laser;
		public var gameSounds:GameSounds;
		
		public function Game() {
			gameSounds = new GameSounds();
			setupIntro();
		}
		
		private function setupIntro():void {
			introScreen = new IntroScreen();
			introScreen.playButton.addEventListener(MouseEvent.CLICK, startGame);
			addChild(introScreen);
			gameSounds.playStart();
			_state = "Intro";
		}
		
		private function startGame(e:MouseEvent):void {
			introScreen.playButton.removeEventListener(MouseEvent.CLICK, startGame);
			removeChild(introScreen);
			gameInit();
		}
		
		private function restartGame(e:MouseEvent):void {
			loseScreen.playButton.removeEventListener(MouseEvent.CLICK, startGame);
			removeChild(loseScreen);
			gameInit()
		}
		
		private function gameInit():void {
			bg1 = new Background();
			addChild(bg1);
			bg2 = new Background();
			bg2.y = -bg1.height;
			addChild(bg2);
			
			Mouse.hide();
			
			_obstacleIndex = this.numChildren;
			for(var i:int=0; i<OBSTACLE_COUNT; i++){
				var obstacle:Obstacle = new Obstacle();
				addChild(obstacle);
			}
			
			player = new Player();
			addChild(player);
			stage.addEventListener(Event.ENTER_FRAME, gameLoop);
			
			laser = new Laser();
			addChild(laser);
			this.addEventListener(PlayerEvent.LASER, laserFired);
			this.addEventListener(PlayerEvent.DEATH, cleanGame);
			this.addEventListener(ObstacleEvent.COMPLETE, explosionComplete);
			
			scoreKeeper = new ScoreKeeper();
			addChild(scoreKeeper);
			
			_playTimeStart = getTimer();
			
			stage.focus = stage;
			
			_state = "Playing";
		}
		
		private function cleanGame(e:PlayerEvent):void {
			stage.removeEventListener(Event.ENTER_FRAME, gameLoop);
			this.removeEventListener(PlayerEvent.LASER, laserFired);
			this.removeEventListener(ObstacleEvent.COMPLETE, explosionComplete);
			Mouse.show();
			
			this.removeChildren();
			
			System.pauseForGCIfCollectionImminent();
			
			_state = "End";
			
			loseScreen = new LoseScreen();
			loseScreen.playButton.addEventListener(MouseEvent.CLICK, restartGame);
			addChild(loseScreen);
			
			gameSounds.playEnd();
		}
		
		private function gameLoop(e:Event):void {
			monitorLaser();
			checkCollision();
			scrollBackground();
			updateScore();
		}
		
		private function monitorLaser():void {
			if(_laserFired){
				laser.y -= 20;
				if(laser.y < -50){
					_laserFired = false;
					laser.x = -2000;
				}else{
					for(var i:int=_obstacleIndex; i<OBSTACLE_COUNT+_obstacleIndex; i++){
						if(this.getChildAt(i).hitTestObject(laser)){
							_laserFired = false;
							laser.x = -2000;
							var obstacle:Obstacle = this.getChildAt(i) as Obstacle;
							
							var explosion:Explosion = new Explosion();
							explosion.x = obstacle.x;
							explosion.y = obstacle.y;
							addChild(explosion);
							obstacle.explode();
							gameSounds.playExplosion();
						}
					}
				}
			}
		}
		
		private function explosionComplete(e:ObstacleEvent):void {
			var explosion:Explosion = e.target as Explosion;
			removeChild(explosion);
		}
		
		private function checkCollision():void {
			for(var i:int=_obstacleIndex; i<OBSTACLE_COUNT+_obstacleIndex; i++){
				if(this.getChildAt(i).hitTestObject(player)){
					var obstacle:Obstacle = this.getChildAt(i) as Obstacle;
					var explosion:Explosion = new Explosion();
					explosion.x = obstacle.x;
					explosion.y = obstacle.y;
					addChild(explosion);
					obstacle.explode();
					player.registerCollision(10);
					gameSounds.playExplosion();
				}
			}
		}
		
		public function laserFired(e:PlayerEvent):void {
			if(!_laserFired){
				laser.x = player.x;
				laser.y = player.y;
				_laserFired = true;
				gameSounds.playLaser();
			}
		}
		
		private function scrollBackground():void {
			if(bg1.y == stage.stageHeight){
				bg1.y = bg2.y-bg1.height+1;
			}else{
				bg1.y += 1;
			}
			if(bg2.y == stage.stageHeight+1){
				bg2.y = bg1.y-bg2.height;
			}else{
				bg2.y += 1;
			}
		}
		
		private function updateScore():void {
			var timeElapsed:String = convertTime(getTimer() - _playTimeStart);
			scoreKeeper.timeOutput.text = timeElapsed;
			scoreKeeper.lifeBar.width = player.getHealth()*2;
		}
		
		private function convertTime(m:int):String {
			var minutes:int = Math.floor((m / (1000 * 60)) % 60);
			var seconds:int = Math.floor(m / 1000 % 60);
			var mS:String;
			var sS:String;
			if (minutes<10) {
				mS = "0"+minutes;
			} else {
				mS = ""+minutes;
			}
			if (seconds<10) {
				sS = "0"+seconds;
			} else {
				sS = ""+seconds;
			}
			var time_string:String = mS+":"+sS;
			return time_string;
		}
		
	}
	
}
