﻿package com.josephlabrecque.actors {
	
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.ui.Keyboard;
	
	import com.josephlabrecque.events.PlayerEvent;
	
	public class Player extends MovieClip {
		
		private const SPEED:int = 12;
		private var _up:Boolean;
		private var _down:Boolean;
		private var _left:Boolean;
		private var _right:Boolean;
		private var _playerHealth:int;
		
		public var playerSkin:MovieClip;
		
		public function Player() {
			this.addEventListener(Event.ADDED_TO_STAGE, addedToStage);
			this.addEventListener(Event.REMOVED_FROM_STAGE, removedFromStage);
			var skinCount:int = playerSkin.totalFrames+1;
			var skinIndex:int = 1 + (skinCount - 1) * Math.random();
			playerSkin.gotoAndStop(skinIndex);
		}
		
		private function addedToStage(e:Event):void {
			this.removeEventListener(Event.ADDED_TO_STAGE, addedToStage);
			this.addEventListener(Event.ENTER_FRAME, playerStatus);
			stage.addEventListener(KeyboardEvent.KEY_DOWN, keyPressed);
			stage.addEventListener(KeyboardEvent.KEY_UP, keyReleased);
			
			_playerHealth = 100;
			this.x = stage.stageWidth/2;
			this.y = stage.stageHeight - this.height - 20;
			this.width = 80;
			this.height = this.width;
		}
		
		private function removedFromStage(e:Event):void {
			this.removeEventListener(Event.ENTER_FRAME, playerStatus);
			this.removeEventListener(Event.REMOVED_FROM_STAGE, removedFromStage);
			stage.removeEventListener(KeyboardEvent.KEY_DOWN, keyPressed);
			stage.removeEventListener(KeyboardEvent.KEY_UP, keyReleased);
		}
		
		private function keyPressed(e:KeyboardEvent):void {
			switch(e.keyCode){
				case Keyboard.UP:
					_up = true;
					break;
				case Keyboard.DOWN:
					_down = true;
					break;
				case Keyboard.LEFT:
					_left = true;
					break;
				case Keyboard.RIGHT:
					_right = true;
					break;
				case Keyboard.SPACE:
					dispatchEvent(new PlayerEvent(PlayerEvent.LASER, true));
					break;
			}
		}
		
		private function keyReleased(e:KeyboardEvent):void {
			switch(e.keyCode){
				case Keyboard.UP:
					_up = false;
					break;
				case Keyboard.DOWN:
					_down = false;
					break;
				case Keyboard.LEFT:
					_left = false;
					break;
				case Keyboard.RIGHT:
					_right = false;
					break;
			}
		}
		
		private function playerStatus(e:Event):void {
			if(_playerHealth <= 0){
				dispatchEvent(new PlayerEvent(PlayerEvent.DEATH, true));
			}else{
				checkMovement();
			}
		}
		
		private function checkMovement():void {
			if(_up == true && this.y > 0){
				this.y -= SPEED;
			}
			if(_down == true && this.y < stage.stageHeight){
				this.y += SPEED;
			}
			if(_left == true && this.x > 0){
				this.x -= SPEED;
			}
			if(_right == true && this.x < stage.stageWidth){
				this.x += SPEED;
			}
		}
		
		public function registerCollision(damage:int):void {
			_playerHealth -= damage;
		}
		
		public function getHealth():int {
			return _playerHealth;
		}
		
	}
	
}
